# -*- coding:utf-8 -*-
# /usr/bin/env python
"""
Date: 2021/12/9 17:20
Desc: Just for test file
"""
import akshare as ak


def test_demo():
    """
    Just for test
    :return: Error
    :rtype: assert result
    """
    cost_living_df = ak.cost_living()
    assert cost_living_df.shape[0] != 0


if __name__ == "__main__":
    test_demo()



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2021/12/13 15:59
Desc: 命令行运行主文件
"""
import os
import sys
from argparse import ArgumentParser
from subprocess import run

import aktools


def get_parser():
    """
    增加 解析属性
    :return:
    :rtype:
    """
    parser = ArgumentParser(description="AKShare's HTTP API Server")
    parser.add_argument(
        "-V",
        "--version",
        action="version",
        version=f"{aktools.__title__} {aktools.__version__}",
    )

    # 添加主机名参数 变量名为host, 必须填
    parser.add_argument(
        "--host",
        action="store",
        dest="host",
        help="Server IP to use for connection",
        default="127.0.0.1",
        type=str,
        required=False,
    )

    # 添加端口号参数 变量名为port, 默认9908, int类型, 非必填
    parser.add_argument(
        "-P",
        "--port",
        action="store",
        dest="port",
        help="port number to use for connection",
        default=8080,
        type=int,
        required=False,
    )

    # 要执行的函数名称
    parser.add_argument(
        "-f",
        "--func",
        help='function name (default: "aktools")',
        choices=["aktools", "slug"],
        default="aktools",
    )
    return parser


def main() -> None:
    """
    主程序
    :return:
    :rtype:
    """
    args = sys.argv[1:]
    parser = get_parser()
    options = parser.parse_args(args)
    # print(options.host, options.port)
    file_address = os.path.dirname(os.path.abspath(aktools.__file__))
    # print(file_address)
    order_str = f"uvicorn run:app --host {options.host} --port {options.port} --app-dir {file_address}"
    print(f"点击打开接口导览：http://{options.host}:{options.port}/docs")
    run(order_str, shell=True)


if __name__ == "__main__":
    main()



# -*- coding:utf-8 -*-
# /usr/bin/env python
"""
Date: 2022/2/22 16:05
Desc: HTTP 模式主文件
"""
import json
import urllib.parse
from importlib import resources
import pathlib


import akshare as ak
from fastapi import APIRouter
from fastapi import Depends, status
from fastapi import Request
from fastapi.responses import JSONResponse, HTMLResponse

from aktools.login.user_login import User, get_current_active_user

app_core = APIRouter()


@app_core.get("/private/{item_id}", description='私人接口', summary="该接口主要提供私密访问来获取数据")
def root(
    request: Request,
    item_id: str,
    current_user: User = Depends(get_current_active_user),
):
    """
    接收请求参数及接口名称并返回 JSON 数据
    此处由于 AKShare 的请求中是同步模式，所以这边在定义 root 函数中没有使用 asyncio 来定义，这样可以开启多线程访问
    :param request: 请求信息
    :type request: Request
    :param item_id: 必选参数; 测试接口名 stock_dxsyl_em 来获取 打新收益率 数据
    :type item_id: str
    :param current_user: 依赖注入，为了进行用户的登录验证
    :type current_user: str
    :return: 指定 接口名称 和 参数 的数据
    :rtype: json
    """
    interface_list = dir(ak)
    decode_params = urllib.parse.unquote(str(request.query_params))
    # print(decode_params)
    if item_id not in interface_list:
        return JSONResponse(
            status_code=status.HTTP_404_NOT_FOUND,
            content={
                "error": "未找到该接口，请升级 AKShare 到最新版本并在文档中确认该接口的使用方式：https://www.akshare.xyz"
            },
        )
    eval_str = decode_params.replace("&", '", ').replace("=", '="') + '"'
    if not bool(request.query_params):
        try:
            received_df = eval("ak." + item_id + f"()")
            if received_df is None:
                return JSONResponse(
                    status_code=status.HTTP_404_NOT_FOUND,
                    content={"error": "该接口返回数据为空，请确认参数是否正确：https://www.akshare.xyz"},
                )
            temp_df = received_df.to_json(orient="records", date_format="iso")
        except KeyError as e:
            return JSONResponse(
                status_code=status.HTTP_404_NOT_FOUND,
                content={
                    "error": f"请输入正确的参数错误 {e}，请升级 AKShare 到最新版本并在文档中确认该接口的使用方式：https://www.akshare.xyz"
                },
            )
        return JSONResponse(status_code=status.HTTP_200_OK, content=json.loads(temp_df))
    else:
        try:
            received_df = eval("ak." + item_id + f"({eval_str})")
            if received_df is None:
                return JSONResponse(
                    status_code=status.HTTP_404_NOT_FOUND,
                    content={"error": "该接口返回数据为空，请确认参数是否正确：https://www.akshare.xyz"},
                )
            temp_df = received_df.to_json(orient="records", date_format="iso")
        except KeyError as e:
            return JSONResponse(
                status_code=status.HTTP_404_NOT_FOUND,
                content={
                    "error": f"请输入正确的参数错误 {e}，请升级 AKShare 到最新版本并在文档中确认该接口的使用方式：https://www.akshare.xyz"
                },
            )
        return JSONResponse(status_code=status.HTTP_200_OK, content=json.loads(temp_df))


@app_core.get("/public/{item_id}", description='公开接口', summary="该接口主要提供公开访问来获取数据")
def root(request: Request, item_id: str):
    """
    接收请求参数及接口名称并返回 JSON 数据
    此处由于 AKShare 的请求中是同步模式，所以这边在定义 root 函数中没有使用 asyncio 来定义，这样可以开启多线程访问
    :param request: 请求信息
    :type request: Request
    :param item_id: 必选参数; 测试接口名 stock_dxsyl_em 来获取 打新收益率 数据
    :type item_id: str
    :return: 指定 接口名称 和 参数 的数据
    :rtype: json
    """
    interface_list = dir(ak)
    decode_params = urllib.parse.unquote(str(request.query_params))
    # print(decode_params)
    if item_id not in interface_list:
        return JSONResponse(
            status_code=status.HTTP_404_NOT_FOUND,
            content={
                "error": "未找到该接口，请升级 AKShare 到最新版本并在文档中确认该接口的使用方式：https://www.akshare.xyz"
            },
        )
    eval_str = decode_params.replace("&", '", ').replace("=", '="') + '"'
    if not bool(request.query_params):
        try:
            received_df = eval("ak." + item_id + f"()")
            if received_df is None:
                return JSONResponse(
                    status_code=status.HTTP_404_NOT_FOUND,
                    content={"error": "该接口返回数据为空，请确认参数是否正确：https://www.akshare.xyz"},
                )
            temp_df = received_df.to_json(orient="records", date_format="iso")
        except KeyError as e:
            return JSONResponse(
                status_code=status.HTTP_404_NOT_FOUND,
                content={
                    "error": f"请输入正确的参数错误 {e}，请升级 AKShare 到最新版本并在文档中确认该接口的使用方式：https://www.akshare.xyz"
                },
            )
        return JSONResponse(status_code=status.HTTP_200_OK, content=json.loads(temp_df))
    else:
        try:
            received_df = eval("ak." + item_id + f"({eval_str})")
            if received_df is None:
                return JSONResponse(
                    status_code=status.HTTP_404_NOT_FOUND,
                    content={"error": "该接口返回数据为空，请确认参数是否正确：https://www.akshare.xyz"},
                )
            temp_df = received_df.to_json(orient="records", date_format="iso")
        except KeyError as e:
            return JSONResponse(
                status_code=status.HTTP_404_NOT_FOUND,
                content={
                    "error": f"请输入正确的参数错误 {e}，请升级 AKShare 到最新版本并在文档中确认该接口的使用方式：https://www.akshare.xyz"
                },
            )
        return JSONResponse(status_code=status.HTTP_200_OK, content=json.loads(temp_df))


def get_pyscript_html(file: str = "akscript.html") -> pathlib.Path:
    """Get path to data "ths.js" text file.
    """
    with resources.path("aktools.assets.html", file) as f:
        data_file_path = f
        return data_file_path


def generate_html_response():
    file_path = get_pyscript_html(file="akscript.html")
    with open(file_path, encoding="utf8") as f:
        html_content = f.read()
    return HTMLResponse(content=html_content, status_code=200)


@app_core.get("/show", response_class=HTMLResponse, description='展示 PyScript', summary="该接口主要展示 PyScript 游览器运行 Python 代码")
def akscript():
    return generate_html_response()



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2022/2/23 20:05
Desc: 主程序入口
"""
import akshare
import uvicorn
from fastapi import FastAPI
from fastapi.middleware.cors import CORSMiddleware

from core import app_core
from login import app_user_login

app = FastAPI(
    title="欢迎来到为 AKShare 打造的 HTTP API 文档",
    description="AKTools 是 AKShare 的 HTTP API 工具, 主要目的是使 AKShare 的数据接口部署到本地从而通过 HTTP 访问来获取所需要的数据",
    version=akshare.__version__,
)

origins = ["*"]

app.add_middleware(
    CORSMiddleware,
    allow_origins=origins,
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
)

app.include_router(app_core, prefix="/api", tags=["数据接口"])
app.include_router(app_user_login, prefix="/auth", tags=["登录接口"])

if __name__ == "__main__":
    uvicorn.run("run:app", host="127.0.0.1", port=8080, reload=True, debug=True)



"""AKTools 是 AKShare 的 HTTP API 工具, 主要目的是使 AKShare 的数据接口部署到本地从而通过 HTTP 访问来获取所需要的数据"""

"""
Changelog

0.0.1: for test
0.0.2: FastAPI test
0.0.3: FastAPI deployment test
0.0.4: add docs
0.0.5: add automate package management
0.0.6: add automate package management success
0.0.7: add automate release test
0.0.8: add automate release test again
0.0.9: add automate success
0.0.10: fix: rename workflows
0.0.11: fix: README.md
0.0.12: fix: README.md
0.0.13: fix: test pull_request
0.0.14: fix: test master push tag
0.0.15: fix: update PYPI info
0.0.16: fix: update PYPI info
0.0.17: fix: update PYPI info
0.0.18: add: add params to interface
0.0.19: fix: rename file name
0.0.20: fix: rename api name
0.0.21: fix: rename api name
0.0.22: fix: fix core module
0.0.23: fix: fix core module
0.0.24: fix: fix chinese character params in api
0.0.25: test: test
0.0.26: test: test
0.0.27: test: test
0.0.28: test: test
0.0.29: test: test
0.0.30: add: add cmd arg parser
0.0.31: add: add cmd arg parser -d
0.0.32: fix: fix docs
0.0.33: fix: fix return result
0.0.34: fix: fix return None
0.0.35: add: add multiple threads support for multiple requests
0.0.36: fix: update PYPI info
0.0.37: fix: update workflow info
0.0.38: fix: fix docs
0.0.39: fix: fix docs
0.0.40: fix: fix docs
0.0.41: fix: fix docs
0.0.42: fix: fix email
0.0.43: add: add support response status code
0.0.44: add: add login support
0.0.45: add: add module format support
0.0.46: fix: fix setup.py
0.0.47: add: add tips
0.0.48: fix: fix docs
0.0.49: add: add FastAPI docs introduction
0.0.50: add: add interface docs
0.0.51: add: add CORS support
0.0.52: add: add PyScript demo
"""

__title__ = "AKTools"
__version__ = "0.0.52"
__author__ = "AKFamily"



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2022/2/23 20:16
Desc: 登录文件
"""
from typing import Optional

from fastapi import APIRouter
from fastapi import Depends, HTTPException, status
from fastapi.security import OAuth2PasswordBearer, OAuth2PasswordRequestForm
from pydantic import BaseModel

app_user_login = APIRouter()

# 用户伪数据库，此处需要修改为 SQLite 数据库
fake_users_db = {
    "akshare": {
        "username": "akshare",
        "full_name": "AKShare AKFamily",
        "email": "akfamily.akshare@gmail.com",
        "hashed_password": "fakehashedakfamily",
        "disabled": False,
    },
}


def fake_hash_password(password: str):
    return "fakehashed" + password


oauth2_scheme = OAuth2PasswordBearer(tokenUrl="auth/token")


class User(BaseModel):
    username: str
    email: Optional[str] = None
    full_name: Optional[str] = None
    disabled: Optional[bool] = None


class UserInDB(User):
    hashed_password: str


def get_user(db, username: str):
    if username in db:
        user_dict = db[username]
        return UserInDB(**user_dict)


def fake_decode_token(token):
    # This doesn't provide any security at all
    # Check the next version
    user = get_user(fake_users_db, token)
    return user


async def get_current_user(token: str = Depends(oauth2_scheme)):
    user = fake_decode_token(token)
    if not user:
        raise HTTPException(
            status_code=status.HTTP_401_UNAUTHORIZED,
            detail="Invalid authentication credentials",
            headers={"WWW-Authenticate": "Bearer"},
        )
    return user


async def get_current_active_user(current_user: User = Depends(get_current_user)):
    if current_user.disabled:
        raise HTTPException(status_code=400, detail="Inactive user")
    return current_user


@app_user_login.post("/token")
async def login(form_data: OAuth2PasswordRequestForm = Depends()):
    """
    用户获取 token 的接口
    """
    user_dict = fake_users_db.get(form_data.username)
    if not user_dict:
        raise HTTPException(status_code=400, detail="Incorrect username or password")
    user = UserInDB(**user_dict)
    hashed_password = fake_hash_password(form_data.password)
    if not hashed_password == user.hashed_password:
        raise HTTPException(status_code=400, detail="Incorrect username or password")

    return {"access_token": user.username, "token_type": "bearer"}



# -*- coding:utf-8 -*-
# /usr/bin/env python
"""
Date: 2022/3/31 17:04
Desc: AKTools 的 PYPI 基本信息文件
"""
import re
import ast

import setuptools


with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()


def get_version_string() -> str:
    """
    Get the aktools version number
    :return: version number
    :rtype: str
    """
    with open("aktools/__init__.py", "rb") as f:
        version_line = re.search(
            r"__version__\s+=\s+(.*)", f.read().decode("utf-8")
        ).group(1)
        return str(ast.literal_eval(version_line))


setuptools.setup(
    name="aktools",
    version=get_version_string(),
    author="AKFamily",
    author_email="akfamily.aktools@gmail.com",
    license="MIT",
    description="AKTools is a tool for AKShare HTTP API!",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/akfamily/aktools",
    packages=setuptools.find_packages(),
    install_requires=[
        "akshare>=1.4.95",
        "fastapi>=0.70.0",
        "uvicorn>=0.16.0",
        "python-multipart>=0.0.5",
    ],
    package_data={"": ["*.py", "*.json", "*.pk", "*.woff"]},
    keywords=[
        "stock",
        "option",
        "futures",
        "bond",
        "index",
        "air",
        "others",
        "finance",
        "spider",
        "quant",
        "quantitative",
        "investment",
        "trading",
        "algotrading",
    ],
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.7",
)



# -*- coding:utf-8 -*-
# /usr/bin/env python
"""
Date: 2021/12/9 19:09
Desc: HTTP 测试
"""
import requests
import pandas as pd

url = "http://127.0.0.1:8080/api/public/public/fund_em_lcx_rank"
params = {
    "symbol": "850335.SI",
}
r = requests.get(url, params=params)
temp_df = pd.DataFrame.from_dict(r.json())
print(temp_df)
